package de.frankmuenster.jameica.depot.db.server;

import java.rmi.RemoteException;
import java.util.Date;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.ValueHistory;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.willuhn.datasource.db.AbstractDBObject;
import de.willuhn.datasource.rmi.ObjectNotFoundException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * @author munstfr
 * 
 */
public class HistoryValueImpl extends AbstractDBObject implements ValueHistory {

	/**
	 * 
	 */
	private static final long serialVersionUID = 4737675122756337592L;

	/**
	 * @throws RemoteException
	 *             - see {@link java.rmi.RemoteException}
	 */
	public HistoryValueImpl() throws RemoteException {
		super();
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#getTableName()
	 */
	@Override
	protected String getTableName() {
		// this is the sql table name.
		return "valueHistory"; //$NON-NLS-1$
	}

	/**
	 * @see de.willuhn.datasource.GenericObject#getPrimaryAttribute()
	 */
	@Override
	public String getPrimaryAttribute() throws RemoteException {
		// our primary attribute is the name.
		return "id"; //$NON-NLS-1$
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#deleteCheck()
	 */
	// protected void deleteCheck() throws ApplicationException {
	// }

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#insertCheck()
	 */
	@Override
	protected void insertCheck() throws ApplicationException {
		try {
			if (getTradingDate() == null) {
				throw new ApplicationException(Settings.getI18n().tr(
						"HistoryValueImpl.tradingDateIsNull")); //$NON-NLS-1$
			}

			if (getAktie() == null) {
				throw new ApplicationException(Settings.getI18n().tr(
						"HistoryValueImpl.stockIsNull")); //$NON-NLS-1$
			}

			if (getAktie().isNewObject()) {
				throw new ApplicationException(Settings.getI18n().tr(
						"HistoryValueImpl.firstStoreStock")); //$NON-NLS-1$
			}

		} catch (RemoteException e) {
			Logger.error(
					Settings.getI18n().tr("HistoryValueImpl.insertCheckFailed"), e); //$NON-NLS-1$
			throw (ApplicationException) new ApplicationException(Settings
					.getI18n().tr(
							Settings.getI18n()
									.tr("HistoryValueImpl.saveFailed"))) //$NON-NLS-1$
					.initCause(e);
		}
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#updateCheck()
	 */
	@Override
	protected void updateCheck() throws ApplicationException {
		// same as insertCheck
		insertCheck();
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#getForeignObject(java.lang.String)
	 */
	@Override
	protected Class<Wertpapier> getForeignObject(final String field)
			throws RemoteException {
		Class<Wertpapier> clas = null;
		// the system is able to resolve foreign keys and loads
		// the according objects automatically. You only have to
		// define which class handles which foreign key.
		if ("aktie_id".equals(field)) { //$NON-NLS-1$
			clas = Wertpapier.class;
		}
		return clas;
	}

	/**
	 * @see ValueHistory#getAktie()
	 */
	@Override
	public Wertpapier getAktie() throws RemoteException {

		Wertpapier wertpapier = null;
		// Yes, we can cast this directly to Wertpapier, because
		// getForeignObject(String)
		// contains the mapping for this attribute.
		try {
			wertpapier = (Wertpapier) getAttribute("aktie_id"); //$NON-NLS-1$
		} catch (ObjectNotFoundException e) {
			Logger.info(Settings.getI18n().tr("Stock not found. ") //$NON-NLS-1$
					+ e.getMessage());
		}
		return wertpapier;
	}

	/**
	 * @see ValueHistory#getTradingDate()
	 */
	@Override
	public Date getTradingDate() throws RemoteException {
		return (Date) getAttribute("tradingDate"); //$NON-NLS-1$
	}

	/**
	 * @see ValueHistory#getTradingPrice()
	 */
	@Override
	public double getTradingPrice() throws RemoteException {
		final Double tradingPrice = (Double) getAttribute("tradingPrice"); //$NON-NLS-1$
		return tradingPrice == null || Double.isNaN(tradingPrice) ? 0.0
				: tradingPrice.doubleValue();
	}

	/**
	 * @see ValueHistory#setAktie(Wertpapier)
	 */
	@Override
	public void setAktie(final Wertpapier wertpapier) throws RemoteException {
		// casting wird in foreignObject bernommen
		setAttribute("aktie_id", wertpapier); //$NON-NLS-1$
	}

	/**
	 * @see ValueHistory#setTradingDate(Date)
	 */
	@Override
	public void setTradingDate(final Date tradingDate) throws RemoteException {
		setAttribute("tradingDate", tradingDate); //$NON-NLS-1$

	}

	/**
	 * @see ValueHistory#setTradingPrice(double)
	 */
	@Override
	public void setTradingPrice(final double tradingPrice)
			throws RemoteException {
		setAttribute("tradingValue", new Double(tradingPrice)); //$NON-NLS-1$

	}

}
