/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.depot.db.server;

import java.rmi.RemoteException;
import java.util.Date;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.OrderHistory;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.willuhn.datasource.db.AbstractDBObject;
import de.willuhn.datasource.rmi.ObjectNotFoundException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * @author munstfr
 * 
 */
public class OrderHistoryImpl extends AbstractDBObject implements OrderHistory {

	/**
	 * Serial UID
	 */
	private static final long serialVersionUID = 8548057261191049423L;

	/**
	 * @throws RemoteException
	 *             e
	 */
	public OrderHistoryImpl() throws RemoteException {
		super();
	}

	/**
	 * @see OrderHistory#getPrimaryAttribute()
	 */
	@Override
	public String getPrimaryAttribute() throws RemoteException {
		return "id"; //$NON-NLS-1$
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#getTableName
	 */
	@Override
	protected String getTableName() {
		return "orderHistory"; //$NON-NLS-1$
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#deleteCheck()
	 */
	// protected void deleteCheck() throws ApplicationException {
	// }

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#insertCheck()
	 */
	@Override
	@SuppressWarnings({ "PMD.ModifiedCyclomaticComplexity",
			"PMD.StdCyclomaticComplexity", "PMD.CyclomaticComplexity" })
	protected void insertCheck() throws ApplicationException {
		try {
			if (getOrderDate() == null) {
				throw new ApplicationException(Settings.getI18n().tr(
						Settings.getI18n().tr("OrderHistoryImpl.enterDate"))); //$NON-NLS-1$
			}

			else if (((Double) getOrderAmount()) == null) {
				throw new ApplicationException(Settings.getI18n().tr(
						Settings.getI18n().tr("OrderHistoryImpl.enterAmount"))); //$NON-NLS-1$
			}

			else if (((Double) getOrderPrice()) == null) {
				throw new ApplicationException(Settings.getI18n().tr(
						Settings.getI18n().tr("OrderHistoryImpl.enterPrice"))); //$NON-NLS-1$
			}

			else if (getAktie() == null) {
				throw new ApplicationException(Settings.getI18n().tr(
						Settings.getI18n().tr("OrderHistoryImpl.chosseStock"))); //$NON-NLS-1$
			}

			else if (getAktie().isNewObject()) {
				throw new ApplicationException(Settings.getI18n().tr(
						Settings.getI18n().tr("OrderHistoryImpl.storeFirst"))); //$NON-NLS-1$
			}

			if (((Double) getOrderPriceInMainCurrency()) == null) {
				setOrderPriceInMainCurrency(0.0d);
			}

			if (((Double) getAdditionalCosts()) == null) {
				setAdditionalCosts(0.0d);
			}

			if (((Double) getAdditionalCostsInMainCurrency()) == null) {
				setAdditionalCostsInMainCurrency(0.0d);
			}

			if (getAdditionalCostsCurrency() == null) {
				setAdditionalCostsCurrency(""); //$NON-NLS-1$
			}

		} catch (RemoteException e) {
			Logger.error(
					Settings.getI18n().tr("OrderHistoryImpl.insertCheckFaild"), e); //$NON-NLS-1$
			throw (ApplicationException) new ApplicationException(Settings
					.getI18n().tr(
							Settings.getI18n().tr(
									"OrderHistoryImpl.unableToStrore"))).initCause(e); //$NON-NLS-1$
		}
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#updateCheck()
	 */
	@Override
	protected void updateCheck() throws ApplicationException {
		// same as insertCheck
		insertCheck();
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#getForeignObject(java.lang.String)
	 */
	@Override
	protected Class<Wertpapier> getForeignObject(final String field)
			throws RemoteException {
		// the system is able to resolve foreign keys and loads
		// the according objects automatically. You only have to
		// define which class handles which foreign key.
		if ("aktie_id".equals(field)) { //$NON-NLS-1$
			return Wertpapier.class; // NOPMD
		}
		return null;
	}

	/**
	 * @see OrderHistory#getAktie()
	 */
	@Override
	public Wertpapier getAktie() throws RemoteException {
		// Yes, we can cast this directly to Wertpapier, because
		// getForeignObject(String)
		// contains the mapping for this attribute.
		try {
			return (Wertpapier) getAttribute("aktie_id"); // NOPMD  //$NON-NLS-1$
		} catch (ObjectNotFoundException e) {
			return null;
		}
	}

	/**
	 * @see OrderHistory#getOrderDate()
	 */
	@Override
	public Date getOrderDate() throws RemoteException {
		return (Date) getAttribute("orderDate"); //$NON-NLS-1$
	}

	/**
	 * @see OrderHistory#getOrderAmount()
	 */
	@Override
	public double getOrderAmount() throws RemoteException {
		final Double orderAmount = (Double) getAttribute("orderAmount"); //$NON-NLS-1$
		return orderAmount == null || Double.isNaN(orderAmount) ? 0.0
				: orderAmount.doubleValue();
	}

	/**
	 * @see OrderHistory#getOrderPrice()
	 */
	@Override
	public double getOrderPrice() throws RemoteException {
		final Double ordnerprice = (Double) getAttribute("orderPrice"); //$NON-NLS-1$
		return ordnerprice == null || Double.isNaN(ordnerprice) ? 0.0
				: ordnerprice.doubleValue();
	}

	/**
	 * @see OrderHistory#getOrderPriceInMainCurrency()
	 */
	@Override
	public double getOrderPriceInMainCurrency() throws RemoteException {
		final Double opInMainCurrency = (Double) getAttribute("orderPriceInMainCurrency"); //$NON-NLS-1$
		return opInMainCurrency == null || Double.isNaN(opInMainCurrency) ? 0.0
				: opInMainCurrency.doubleValue();
	}

	/**
	 * @see OrderHistory#getAdditionalCosts()
	 */
	@Override
	public double getAdditionalCosts() throws RemoteException {
		final Double additionalCosts = (Double) getAttribute("additionalCosts"); //$NON-NLS-1$
		return additionalCosts == null || Double.isNaN(additionalCosts) ? 0.0
				: additionalCosts.doubleValue();
	}

	/**
	 * @see OrderHistory#getAdditionalCostsInMainCurrency()
	 */
	@Override
	public double getAdditionalCostsInMainCurrency() throws RemoteException {
		final Double acInMainCurrency = (Double) getAttribute("additionalCostsInMainCurrency"); //$NON-NLS-1$
		return acInMainCurrency == null || Double.isNaN(acInMainCurrency) ? 0.0
				: acInMainCurrency.doubleValue();
	}

	/**
	 * @see OrderHistory#getAdditionalCostsCurrency()
	 */
	@Override
	public String getAdditionalCostsCurrency() throws RemoteException {
		return (String) getAttribute("additionalCostsCurrency"); //$NON-NLS-1$
	}

	/**
	 * @see OrderHistory#setAktie(Wertpapier)
	 */
	@Override
	public void setAktie(final Wertpapier wertpapier) throws RemoteException {
		// casting wird in foreignObject bernommen
		setAttribute("aktie_id", wertpapier); //$NON-NLS-1$
	}

	/**
	 * @see OrderHistory#setOrderDate(Date)
	 */
	@Override
	public void setOrderDate(final Date orderDate) throws RemoteException {
		setAttribute("orderDate", orderDate); //$NON-NLS-1$

	}

	/**
	 * @see OrderHistory#setOrderAmount(double)
	 */
	@Override
	public void setOrderAmount(final double orderAmount) throws RemoteException {
		setAttribute("orderAmount", new Double(orderAmount)); //$NON-NLS-1$

	}

	/**
	 * @see OrderHistory#setOrderPrice(double)
	 */
	@Override
	public void setOrderPrice(final double orderPrice) throws RemoteException {
		setAttribute("orderPrice", new Double(orderPrice)); //$NON-NLS-1$

	}

	/**
	 * @see OrderHistory#setOrderPriceInMainCurrency(double)
	 */
	@Override
	public void setOrderPriceInMainCurrency(final double orderPrice)
			throws RemoteException {
		setAttribute("orderPriceInMainCurrency", new Double(orderPrice)); //$NON-NLS-1$

	}

	/**
	 * @see OrderHistory#setAdditionalCosts(double)
	 */
	@Override
	public void setAdditionalCosts(final double additionalCosts)
			throws RemoteException {
		setAttribute("additionalCosts", new Double(additionalCosts)); //$NON-NLS-1$

	}

	/**
	 * @see OrderHistory#setAdditionalCostsInMainCurrency(double)
	 */
	@Override
	public void setAdditionalCostsInMainCurrency(final double additionalCosts)
			throws RemoteException {
		setAttribute("additionalCostsInMainCurrency", new Double( //$NON-NLS-1$
				additionalCosts));

	}

	/**
	 * @see OrderHistory#setAdditionalCostsCurrency(String)
	 */
	@Override
	public void setAdditionalCostsCurrency(final String additionalCosts)
			throws RemoteException {
		setAttribute("additionalCostsCurrency", additionalCosts); //$NON-NLS-1$

	}

	/**
	 * @see OrderHistory#getComment()
	 */

	@Override
	public String getComment() throws RemoteException {
		return (String) getAttribute("comment"); //$NON-NLS-1$
	}

	/**
	 * @see OrderHistory#setComment(String)
	 */

	@Override
	public void setComment(final String comment) throws RemoteException {
		setAttribute("comment", comment); //$NON-NLS-1$
	}

}
