/*
 * Copyright 2014 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.gui.action;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.JaDepotDBService;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.WertpapierInfoBean;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.YahooFinanceAPI;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.YahooFinanceException;
import de.frankmuenster.jameica.depot.gui.action.listener.WertpapierAktualisiertListener;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * @author munstfr
 * 
 */
public class WertpapierAktualisiernAction implements Action {

	/**
	 * Die Listener fr einen QrCodeFound-Event
	 */
	private final transient Set<WertpapierAktualisiertListener> listeners = new CopyOnWriteArraySet<WertpapierAktualisiertListener>();

	/**
	 * Standart Konstruktor
	 */
	public WertpapierAktualisiernAction() {
		// absichtlich leer
	}

	/**
	 * Fgt eine Listener hinzu
	 * 
	 * @param listener
	 *            der hinzugefgt werden soll
	 */
	public final void addListener(final WertpapierAktualisiertListener listener) {
		listeners.add(listener);
	}

	/**
	 * Entfernd einen Listener aus der Liste
	 * 
	 * @param listener
	 *            der entfernt werden soll
	 */
	public final void removeListener(
			final WertpapierAktualisiertListener listener) {
		listeners.remove(listener);
	}

	/**
	 * Benachrichtigen der registrieretn Listeners mit bergabe des gefundenen
	 * Strings
	 */
	private final void notifyListeners() {
		for (final WertpapierAktualisiertListener listener : listeners) {
			listener.notifyWertpapiereAktualisiert();
		}
	}

	/**
	 * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
	 */
	@Override
	public void handleAction(final Object context) throws ApplicationException {

		try {
			if (context instanceof Wertpapier) {
				final Wertpapier wertpapier = (Wertpapier) context;
				reloadWertpapierFromWeb(wertpapier);
			} else {
				reloadAlleWerteFromWeb();
			}

			notifyListeners();
		} catch (RemoteException e) {
			Logger.error(e.getMessage());
			throw (ApplicationException) new ApplicationException(Settings
					.getI18n().tr("WertpapierAktualisiernAction.updateError")).initCause(e); //$NON-NLS-1$
		}
	}

	private boolean reloadWertpapierFromWeb(final Wertpapier wertpapier) {

		boolean isLoadOK = false;
		String symbol = null;

		try {
			symbol = wertpapier.getSymbol();
		} catch (RemoteException e) {
			Logger.error(Settings.getI18n().tr(
					"WertpapierAktualisiernAction.symbolNotFound")); //$NON-NLS-1$
			if (Logger.isLogging(Level.DEBUG)) {
				Logger.debug(e.getMessage());
			}
		}
		if (StringUtils.isNotEmpty(symbol)) {
			try {
				final WertpapierInfoBean wpi = YahooFinanceAPI.getInstance()
						.getWertpapierInfo(symbol, false);
				if (wpi != null) {
					wertpapier.parseWertpapier(wpi);
					wertpapier.store();
					Logger.debug(Settings.getI18n().tr(
							"WertpapierAktualisiernAction.success", symbol)); //$NON-NLS-1$
					isLoadOK = true;
				}
			} catch (ApplicationException e) {
				Logger.error(Settings.getI18n().tr(
						"WertpapierAktualisiernAction.storeError", //$NON-NLS-1$
						symbol));
				Logger.debug(e.getMessage());
			} catch (YahooFinanceException e) {
				Logger.debug(e.getMessage());
			} catch (RemoteException e) {
				Logger.error(Settings.getI18n().tr(
						"WertpapierAktualisiernAction.remoteError", symbol)); //$NON-NLS-1$
				Logger.debug(e.getMessage());
			}
		}
		return isLoadOK;
	}

	private void reloadAlleWerteFromWeb() throws RemoteException {

		JaDepotDBService service;

		service = (JaDepotDBService) Settings.getDBService();
		final DBIterator<?> list = service.createList(Wertpapier.class);

		final ArrayList<String> errMessage = new ArrayList<String>();

		while (list.hasNext()) {
			final Wertpapier wertpapier = (Wertpapier) list.next();
			String symbol;
			try {
				symbol = wertpapier.getSymbol();
			} catch (RemoteException e) {
				Logger.error(Settings.getI18n().tr(
						"WertpapierAktualisiernAction.symbolNotFound")); //$NON-NLS-1$
				errMessage.add(Settings.getI18n().tr(
						"WertpapierAktualisiernAction.symbolNotFoundMsg")); //$NON-NLS-1$
				continue;
			}
			if (!reloadWertpapierFromWeb(wertpapier)) {
				errMessage.add(symbol);
			}
		}
		if (errMessage.isEmpty()) {
			final String msg = Settings.getI18n().tr(
					"WertpapierAktualisiernAction.successful"); //$NON-NLS-1$
			Application.getMessagingFactory().sendMessage(
					new StatusBarMessage(msg, StatusBarMessage.TYPE_SUCCESS));
		} else {
			final StringBuffer msg = new StringBuffer(Settings.getI18n().tr(
					"WertpapierAktualisiernAction.stocks")); //$NON-NLS-1$
			msg.append(errMessage.get(0));
			for (int i = 1; i < errMessage.size(); i++) {
				msg.append(',').append(errMessage.get(i));
			}
			msg.append(Settings.getI18n().tr(
					"WertpapierAktualisiernAction.notFoundInYahoo")); //$NON-NLS-1$
			Application.getMessagingFactory().sendMessage(
					new StatusBarMessage(msg.toString(),
							StatusBarMessage.TYPE_INFO));
		}
	}
}
