/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.action;

import java.rmi.RemoteException;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.OrderHistory;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Enternt ein Wertpapier aus Datenbank
 * 
 * @author munstfr
 * @version 1.0
 */
public class WertpapierDeleteAction implements Action {

	/**
	 * Standart Konstruktor
	 */
	public WertpapierDeleteAction() {
		// absichtlich leer
	}

	/**
	 * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
	 */
	@Override
	public void handleAction(final Object context) throws ApplicationException {

		// check if the context is a project
		if (!(context instanceof Wertpapier)) {
			throw new ApplicationException(Settings.getI18n().tr(
					"WertpapierDeleteAction.chooseStock")); //$NON-NLS-1$
		}

		final Wertpapier wertpapier = (Wertpapier) context;

		try {
			// Sicherheitsabfrage
			final String question = Settings.getI18n().tr(
					"WertpapierDeleteAction.reallyDelete"); //$NON-NLS-1$
			if (Application.getCallback().askUser(question)) {

				try {
					final DBIterator<?> orders = wertpapier.getOrderHistory();

					while (orders.hasNext()) {
						final OrderHistory orderHistory = (OrderHistory) orders
								.next();
						orderHistory.delete();
					}

				} catch (RemoteException e1) {
					Logger.warn(e1.getMessage());
				}

				wertpapier.delete();
				// Send Status update message
				Application.getMessagingFactory().sendMessage(
						new StatusBarMessage(Settings.getI18n().tr(
								"WertpapierDeleteAction.success"), //$NON-NLS-1$
								StatusBarMessage.TYPE_SUCCESS));
			}

		} catch (Exception e) { // NOPMD
			Logger.error(
					Settings.getI18n().tr("WertpapierDeleteAction.error"), e); //$NON-NLS-1$
			throw (ApplicationException) new ApplicationException(Settings
					.getI18n().tr("WertpapierDeleteAction.error")).initCause(e); //$NON-NLS-1$
		}
	}

}
