/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.gui.controller;

import java.rmi.RemoteException;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.WertpapierHistory;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.YahooFinanceException;
import de.frankmuenster.jameica.depot.gui.action.OrderDetailAction;
import de.frankmuenster.jameica.depot.gui.action.listener.OrderDeleteListener;
import de.frankmuenster.jameica.depot.gui.action.listener.WertpapierAktualisiertListener;
import de.frankmuenster.jameica.depot.gui.input.CurrencyInput;
import de.frankmuenster.jameica.depot.gui.input.PeriodChooserInput;
import de.frankmuenster.jameica.depot.gui.input.ZeitauswahlSchieberInput;
import de.frankmuenster.jameica.depot.gui.input.ZeitauswahlSchieberInputListener;
import de.frankmuenster.jameica.depot.gui.menu.OrderListeMenu;
import de.frankmuenster.jameica.depot.gui.parts.DiagrammAnzeigeAuswahlPart;
import de.frankmuenster.jameica.depot.gui.parts.OrderListePart;
import de.frankmuenster.jameica.depot.gui.parts.WertpapierDetailsPart;
import de.frankmuenster.jameica.depot.gui.parts.WertpapierPerformanzPart;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.AbstractGrafik;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.kursgrafik.KursGrafik;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.gui.AbstractControl;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.input.CheckboxInput;
import de.willuhn.jameica.gui.input.DateInput;
import de.willuhn.jameica.gui.input.DecimalInput;
import de.willuhn.jameica.gui.input.Input;
import de.willuhn.jameica.gui.input.RadioInput;
import de.willuhn.jameica.gui.input.TextAreaInput;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.gui.parts.Button;
import de.willuhn.jameica.gui.parts.ContextMenu;
import de.willuhn.jameica.gui.parts.TablePart;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

//TODO: Grafikwerkzeuge als eigenstndiges Part mit eigenem Event-Handler. Ergebnisse sollten dann in einem Event argument and Listener verteilt werden.

/**
 * Komplexer Wertpapier Anzeige-Kontroller
 * 
 * @author munstfr
 * 
 */
@SuppressWarnings({ "PMD.ExcessiveImports", "PMD.GodClass", "PMD.TooManyFields" })
public class WertpapierDetailController extends AbstractControl implements
		WertpapierAktualisiertListener, OrderDeleteListener {

	/**
	 * Das Wertpapier das wird kontrollieren wollen
	 */
	private transient Wertpapier wertpapier;

	/**
	 * Symbol-Eingabefeld
	 */
	private transient TextInput symbol;

	/**
	 * Name-Eingabefeld
	 */
	private transient TextInput name;

	/**
	 * Kommentar-Eingabefeld
	 */
	private transient TextAreaInput comment;

	/**
	 * Die Performanz-Anzeige
	 */
	private transient WertpapierPerformanzPart performanzPart;

	/**
	 * Datum-Eingabefeld
	 * */
	private transient DateInput datum;

	/**
	 * Kurswert-Eingabefeld
	 */
	private transient DecimalInput kurs;

	/**
	 * W&auml;hrungs-Eingabefeld
	 */
	private transient CurrencyInput waehrung;

	/**
	 * OrderListe
	 */
	private transient TablePart orderHistoryList;

	/**
	 * Grafikwerkzeug - Zeitschhieber TODO: in externe Klasse
	 */
	private transient ZeitauswahlSchieberInput zeitAuswahl;

	/**
	 * Grafikwerkzeug - ZeitAuswahlButtons TODO: in externe Klasse
	 */
	private transient PeriodChooserInput zeitAuswahlBttns;

	/**
	 * Grafikwerkzeug - ZeigeDax TODO: in externe Klasse
	 */
	private transient CheckboxInput showDax;

	/**
	 * Grafikwerkzeug - Zeige als Kerzengrafik TODO: in externe Klasse
	 */
	private transient CheckboxInput showCandle;

	/**
	 * Container f&uuml;r Grafikwerzeuge TODO: Als vollwertige Klasse mit
	 * Event-Handler erweitern
	 */
	private transient DiagrammAnzeigeAuswahlPart grafikWerkzeuge;

	/**
	 * Kursgrafik
	 */
	private transient KursGrafik grafik;

	/**
	 * Details zum Wertpapier TODO: Check ob als vollwertige Klasse
	 * implementiert werden kann.
	 */
	private transient WertpapierDetailsPart wpDetailsPart;

	/**
	 * Historische Werte des Wertpapiers
	 */
	private transient WertpapierHistory wertpapierHistory;

	/**
	 * Historische DAX-Werte
	 */
	private transient WertpapierHistory daxHistory;

	/**
	 * Sind historische Werte verf&uuml;gbar
	 */
	private transient boolean histDataAvailable;

	/**
	 * Buttonm zum Speichern
	 */
	private transient Button saveWertpapier;

	/**
	 * @param view
	 * @throws RemoteException
	 */
	public WertpapierDetailController(final AbstractView view)
			throws RemoteException {
		super(view);

		final String symbolName = getWertPapier().getSymbol();
		loadData(symbolName);
		loadDaxData();
		if (wertpapierHistory != null && daxHistory != null) {
			histDataAvailable = true;
		}
	}

	/**
	 * Liefert das aktuelle Wertpapier.
	 * 
	 * @return das WertPapier.
	 */
	private Wertpapier getWertPapier() {
		if (wertpapier == null) {
			wertpapier = (Wertpapier) getCurrentObject();
		}
		return wertpapier;
	}

	/**
	 * Liefert true, falls die Historiedaten geladen werden konnten.
	 * 
	 * @return true wenn die Kurs- und Daxhistorie geladen werden konnte, false
	 *         sonst.
	 */
	public boolean isHistoDataAvailable() {
		return histDataAvailable;
	}

	/**
	 * Returns the wertpapierDetails Part for the symbol name.
	 * 
	 * @return part wertpapierDetails part
	 * @throws RemoteException
	 */
	public WertpapierDetailsPart getWertpapierDetails() throws RemoteException {
		if (wpDetailsPart == null) {
			wpDetailsPart = new WertpapierDetailsPart(wertpapier, this);
		}
		return wpDetailsPart;
	}

	/**
	 * Returns the input field for the symbol name.
	 * 
	 * @return input field.
	 * @throws RemoteException
	 */
	public Input getSymbol() throws RemoteException {
		if (symbol == null) { // "255" is the maximum length for this input
								// field.
			symbol = new TextInput(getWertPapier().getSymbol(), 255);
			symbol.setMandatory(true);
			symbol.setName(Settings.getI18n().tr(
					"WertpapierDetailController.symbol")); //$NON-NLS-1$
		}
		return symbol;
	}

	/**
	 * Returns the input field for the project name.
	 * 
	 * @return input field.
	 * @throws RemoteException
	 */
	public Input getName() throws RemoteException {
		if (name == null) { // "255" is the maximum length for this input field.
			name = new TextInput(getWertPapier().getName(), 255);
			name.setMandatory(true);
			name.setName(Settings.getI18n().tr(
					"WertpapierDetailController.name")); //$NON-NLS-1$
		}
		return name;
	}

	/**
	 * Liefert ein {@link DateInput} filed mit den LastTrade Feld der Aktie aus
	 * der Datenbank.
	 * 
	 * @return DateInput field
	 * 
	 * @throws RemoteException
	 */
	public DateInput getDatum() throws RemoteException {
		if (datum == null) {
			final DateFormat format = new SimpleDateFormat("dd.MM.yyyy", //$NON-NLS-1$
					Locale.GERMAN);
			final Date lastTradeDate = getWertPapier().getLastTradeDate();
			datum = new DateInput(lastTradeDate, format);
			datum.setTitle(Settings.getI18n().tr(
					"WertpapierDetailController.lastTradeDate")); //$NON-NLS-1$
			datum.setText(Settings.getI18n().tr(
					"WertpapierDetailController.choseDate")); //$NON-NLS-1$
		}
		return datum;
	}

	/**
	 * Liefert ein {@link CurrencyInput} field mit der W&auml;hrung der Aktie
	 * aus der Datenbank.
	 * 
	 * @return DateInput field
	 * 
	 * @throws RemoteException
	 */
	public CurrencyInput getWaehrung() throws RemoteException {
		if (waehrung == null) {
			waehrung = new CurrencyInput(getWertPapier().getCurrency());
		}
		return waehrung;
	}

	/**
	 * Liefert ein {@link DecimalInput} mit dem Kurswert der Aktie aus der
	 * Datenbank.
	 * 
	 * @return DecimalInput field.
	 * @throws RemoteException
	 */
	public DecimalInput getLetzterKursWert() throws RemoteException {
		if (kurs == null) {
			final DecimalFormat format = new DecimalFormat("###,###,##0.00"); //$NON-NLS-1$
			kurs = new DecimalInput(getWertPapier().getLastTradePriceOnly(),
					format);
		}
		return kurs;
	}

	/**
	 * Liefert das OrderHistoryPart
	 * 
	 * @return das OrderHistoryPart
	 * @throws RemoteException
	 */
	public Part getOrderHistoryList() throws RemoteException {

		if (orderHistoryList == null) {
			final DBIterator<?> orderHistory = getWertPapier()
					.getOrderHistory();
			orderHistoryList = new OrderListePart(orderHistory,
					new OrderDetailAction());

			// Kontext Men hinzufgen
			final ContextMenu menu = new OrderListeMenu(wertpapier, this);
			orderHistoryList.setContextMenu(menu);

		}
		return orderHistoryList;

	}

	/**
	 * Liefert das Kommentarfeld
	 * 
	 * @return das Kommentarfeld
	 * @throws RemoteException
	 */
	public Input getComment() throws RemoteException {
		if (comment == null) {
			comment = new TextAreaInput(getWertPapier().getComment());
		}
		return comment;
	}

	/**
	 * Liefert den Chart
	 * 
	 * @return den Chart
	 */
	public AbstractGrafik getGrafik() {

		if (grafik == null && wertpapierHistory != null && daxHistory != null) {

			grafik = new KursGrafik();
			grafik.setKursDaxDaten(wertpapierHistory, daxHistory);

			if (zeitAuswahl == null) {
				grafik.setFromDate(wertpapierHistory.getFromDate());
				grafik.setToDate(wertpapierHistory.getToDate());
			} else {

				grafik.setFromDate(zeitAuswahl.getLower());
				grafik.setToDate(zeitAuswahl.getUpper());
			}

			grafik.setShowAsCandleStick(false);
			grafik.setShowDax(false);
		}
		return grafik;

	}

	/**
	 * Liefert das Zeitauswahl-Part
	 * 
	 * @return das Zeitauswahl-Part
	 */
	public Part getZeitauswahl() {
		if (zeitAuswahl == null) {
			zeitAuswahl = new ZeitauswahlSchieberInput();
			// Zeitachse setzten und das letzte Jahr auswhlen
			zeitAuswahl.setScaling(this.wertpapierHistory.getFromDate(),
					this.wertpapierHistory.getToDate(), 1, 10);
			zeitAuswahl.selectLastYear();
			// Platz fr den Kommentar schaffen
			zeitAuswahl.setComment(""); //$NON-NLS-1$
			zeitAuswahl.addListener(new ZeitAuswahlListener());
		}
		return zeitAuswahl;

	}

	/**
	 * Liefert die Grafikwerkzeuge in einem Part
	 * 
	 * @return DiagrammAnzeigeAuswahlPart (Grafikwerkzeuge)
	 */
	public DiagrammAnzeigeAuswahlPart getGrafikWerkzeuge() {

		if (grafikWerkzeuge == null && wertpapierHistory != null) {

			grafikWerkzeuge = new DiagrammAnzeigeAuswahlPart();
			if (daxHistory == null) {
				grafikWerkzeuge.setDax(null);
			} else {
				grafikWerkzeuge.setDax(getShowDax());
			}
			grafikWerkzeuge
					.setPeriod((PeriodChooserInput) getZeitauswahlButtons());
			grafikWerkzeuge.setHighLow(getShowCandle());
		}
		return grafikWerkzeuge;

	}

	private Part getZeitauswahlButtons() {

		if (zeitAuswahlBttns == null) {
			zeitAuswahlBttns = new PeriodChooserInput();
			zeitAuswahlBttns.addListener(new ZeitAuswahlButtonListener());
		}
		return zeitAuswahlBttns;

	}

	private CheckboxInput getShowDax() {

		if (showDax == null) {
			showDax = new CheckboxInput(false);
			showDax.setName("DAX"); //$NON-NLS-1$
			showDax.setEnabled(true);

			showDax.addListener(new Listener() {

				/**
				 * @see org.eclipse.swt.widgets.Listener#handleEvent(Event
				 *      event)
				 */
				@Override
				public void handleEvent(final Event event) {
					Logger.warn("DAX:" + showDax.getValue()); //$NON-NLS-1$
					if (event.type == SWT.Selection) {
						grafik.setShowDax((Boolean) showDax.getValue());
						if ((Boolean) showDax.getValue()) {
							showCandle.setValue(false);
						}

						grafik.reload();
					}

				}
			});
		}
		return showDax;
	}

	private CheckboxInput getShowCandle() {

		if (showCandle == null) {
			showCandle = new CheckboxInput(false);
			showCandle.setName(Settings.getI18n().tr(
					"WertpapierDetailController.candle")); //$NON-NLS-1$

			showCandle.addListener(new Listener() {

				/**
				 * @see org.eclipse.swt.widgets.Listener#handleEvent(Event
				 *      event)
				 */
				@Override
				public void handleEvent(final Event event) {
					Logger.warn("LowHigh:" + showCandle.getValue()); //$NON-NLS-1$
					if (event.type == SWT.Selection) {
						grafik.setShowAsCandleStick((Boolean) showCandle
								.getValue());
						if ((Boolean) showCandle.getValue()) {
							showDax.setValue(false);
						}
						grafik.reload();
					}
				}
			});
		}
		return showCandle;
	}

	private void loadData(final String symbol) {

		if (StringUtils.isEmpty(symbol)) {
			wertpapierHistory = null; // NOPMD
		} else {
			try {
				if (getWertPapier().isInYahoo()) {
					wertpapierHistory = new WertpapierHistory(symbol);
				}
			} catch (YahooFinanceException e) {
				wertpapierHistory = null; // NOPMD
			} catch (RemoteException e) {
				wertpapierHistory = null; // NOPMD
			}
		}
	}

	private void loadDaxData() {

		try {
			daxHistory = new WertpapierHistory("^GDAXI"); //$NON-NLS-1$
		} catch (YahooFinanceException e) {
			daxHistory = null; // NOPMD
		}
	}

	/**
	 * Private Listener Klasse, die auf den ZeitauswahlSchieber reagiert.
	 * 
	 * @author munstfr
	 * 
	 */
	private class ZeitAuswahlListener implements
			ZeitauswahlSchieberInputListener {

		/**
		 * Standardkonstruktor
		 */
		public ZeitAuswahlListener() {
			// absichtlich leer
		}

		/**
		 * @see ZeitauswahlSchieberInputListener#rangeChanged()
		 */
		@Override
		public void rangeChanged() {
			Logger.warn(Settings.getI18n().tr(
					"WertpapierDetailController.timeRangeChnaged")); //$NON-NLS-1$
			grafik.setFromDate(zeitAuswahl.getLower());
			grafik.setToDate(zeitAuswahl.getUpper());
			grafik.reload();
		}

	}

	/**
	 * Private Listener Klasse, die auf den ZeitAuswahlButton reagiert.
	 * 
	 * @author munstfr
	 * 
	 */
	private class ZeitAuswahlButtonListener implements Listener {

		/**
		 * Standardkonstruktor
		 */
		public ZeitAuswahlButtonListener() {
			// absichtlich leer
		}

		/**
		 * @see ZeitAuswahlButtonListener#handleEvent(Event)
		 */
		@Override
		public void handleEvent(final Event event) {

			if (event.type != SWT.FocusIn && event.type != SWT.FocusOut
					&& event.widget instanceof org.eclipse.swt.widgets.Button) {

				final int tage = (Integer) RadioInput
						.getValue(PeriodChooserInput.GROUP_ID);
				final int toDate = zeitAuswahl.getMaximun();

				zeitAuswahl.setUpper(toDate);
				zeitAuswahl.setLower(toDate - tage);

				grafik.setFromDate(zeitAuswahl.getLower());
				grafik.setToDate(zeitAuswahl.getUpper());
				grafik.reload();
			}
		}
	}

	/**
	 * Liefert den Speichern-Button
	 * 
	 * @return den Speichern-Button
	 */
	public Button getSave() {

		if (saveWertpapier == null) {
			saveWertpapier = new Button(Settings.getI18n().tr(
					Settings.getI18n().tr("WertpapierDetailController.save")), //$NON-NLS-1$
					new Action() {

						/**
						 * @see de.willuhn.jameica.gui.Action#handleAction(Object
						 *      context)
						 */
						@Override
						public void handleAction(final Object context)
								throws ApplicationException {
							handleSaveWertpapier();
						}
					});
		}
		return saveWertpapier;
	}

	/**
	 * Liefert das WertpapierPerformanzPart.
	 * 
	 * @return das Performanz-Part
	 */
	public WertpapierPerformanzPart getPerformanzPart() {
		if (performanzPart == null) {
			performanzPart = new WertpapierPerformanzPart(wertpapier);
		}
		return performanzPart;
	}

	private void handleSaveWertpapier() {

		try {

			if (!wertpapier.isInYahoo()) {
				wertpapier.setLastTradeDate((Date) getDatum().getValue());
				wertpapier.setLastTradePriceOnly((Double) getLetzterKursWert()
						.getValue());
				wertpapier.setCurrency((String) getWaehrung().getValue());
			}
			wertpapier.setComment((String) getComment().getValue());
			wertpapier.store();
			Application.getMessagingFactory().sendMessage(
					new StatusBarMessage(Settings.getI18n().tr(
							"WertpapierDetailController.commentSaveSuccess"), //$NON-NLS-1$
							StatusBarMessage.TYPE_SUCCESS));
		} catch (RemoteException | ApplicationException e) {
			final String message = Settings.getI18n().tr(
					"WertpapierDetailController.commentSaveError"); //$NON-NLS-1$
			Application.getMessagingFactory().sendMessage(
					new StatusBarMessage(message, StatusBarMessage.TYPE_ERROR));
			Logger.error(e.getMessage());
		}
		try {
			getWertpapierDetails().update();
		} catch (RemoteException e) {
			final String message = Settings.getI18n().tr(
					"WertpapierDetailController.updateViewFailed"); //$NON-NLS-1$
			Application.getMessagingFactory().sendMessage(
					new StatusBarMessage(message, StatusBarMessage.TYPE_ERROR));
		}

	}

	private void updateFelder() {
		// Wertpapierdeatail Felder aktualisieren
		if (wpDetailsPart != null) {
			wpDetailsPart.update();
		}
		if (performanzPart != null) {
			performanzPart.update();
		}
	}

	/**
	 * @see WertpapierAktualisiertListener#notifyWertpapiereAktualisiert()
	 */
	@Override
	public void notifyWertpapiereAktualisiert() {
		updateFelder();
	}

	/**
	 * @see de.frankmuenster.jameica.depot.gui.action.listener.OrderDeleteListener#notifyOrderDeleted()
	 */
	@Override
	public void notifyOrderDeleted() {
		updateFelder();
	}

}
