/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.input;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;

import de.frankmuenster.jameica.depot.Settings;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.input.Input;
import de.willuhn.jameica.gui.input.RadioInput;
import de.willuhn.jameica.gui.util.Color;
import de.willuhn.jameica.gui.util.Font;

/**
 * 
 * @author munstfr
 * 
 *         Abgeleitet u.a. von der MultiInput-Klasse von Jameica
 */
public class PeriodChooserInput implements Input {

	/**
	 * Context-Parameter f&uuml;r die Wichtung der Breite.
	 * (jameica.multiinput.weight)
	 */
	private final static String DATA_WEIGHT = "jameica.multiinput.weight"; //$NON-NLS-1$

	/**
	 * Gruppen ID f&uuml;r die Radio-Buttons (periodChooser)
	 */
	public static final String GROUP_ID = "periodChooser"; //$NON-NLS-1$

	/**
	 * Tooltip key f&uuml;r Dat Feld
	 */
	private final static String TOOLTIP = "tooltip"; //$NON-NLS-1$

	/**
	 * Periodenwert f&uuml;r ein Woche (7)
	 */
	public final static int PERIOD_1W = 7;

	/**
	 * Periodenwert f&uuml;r einen Monat (30)
	 */
	public final static int PERIOD_1M = 30;

	/**
	 * Periodenwert f&uuml;r 3 Monate (90)
	 */
	public final static int PERIOD_3M = 90;

	/**
	 * Periodenwert f&uuml;r 6 Monate (180)
	 */
	public final static int PERIOD_6M = 180;

	/**
	 * Periodenwert f&uuml;r ein Jahr (365)
	 */
	public final static int PERIOD_1J = 365;

	/**
	 * Periodenwert f&uuml;r 2 Jaher (730)
	 */
	public final static int PERIOD_2J = 730;

	/**
	 * Periodenwert f&uuml;r 5 Jahr5 (1825)
	 */
	public final static int PERIOD_5J = 1825;

	/**
	 * Map f&uuml; belibige Nutzer Daten
	 */
	private final Map<String, Object> data = new ConcurrentHashMap<String, Object>();

	/**
	 * Liste aller Input-Objeckte
	 */
	private final transient List<Input> inputs = new ArrayList<Input>();

	/**
	 * Unser Composite auf dem wir malen.
	 */
	private transient Composite composite;

	/**
	 * Die Labelbezeichung
	 */
	private String name;

	/**
	 * Kommentartext
	 */
	private transient String comment;

	/**
	 * Kommentar-Label
	 */
	private transient Label commentLabel;

	/**
	 * Konstruktor
	 * 
	 */
	public PeriodChooserInput() {

		RadioInput radioInput = new RadioInput(GROUP_ID, PERIOD_1M);
		radioInput.setName(Settings.getI18n().tr(
				"PeriodChooserInput.oneMonthName")); //$NON-NLS-1$
		radioInput.setData(TOOLTIP,
				Settings.getI18n().tr("PeriodChooserInput.oneMonthTooltip")); //$NON-NLS-1$
		inputs.add(radioInput);

		radioInput = new RadioInput(GROUP_ID, PERIOD_3M);
		radioInput.setName(Settings.getI18n().tr(
				"PeriodChooserInput.threeMonthName")); //$NON-NLS-1$
		radioInput.setData(TOOLTIP,
				Settings.getI18n().tr("PeriodChooserInput.threeMonthTooltip")); //$NON-NLS-1$
		inputs.add(radioInput);

		radioInput = new RadioInput(GROUP_ID, PERIOD_6M);
		radioInput.setName(Settings.getI18n().tr(
				"PeriodChooserInput.sixMonthName")); //$NON-NLS-1$
		radioInput.setData(TOOLTIP,
				Settings.getI18n().tr("PeriodChooserInput.sixMonthTooltip")); //$NON-NLS-1$
		inputs.add(radioInput);

		radioInput = new RadioInput(GROUP_ID, PERIOD_1J);
		radioInput.setName(Settings.getI18n().tr(
				"PeriodChooserInput.oneYearName")); //$NON-NLS-1$
		radioInput.setData(TOOLTIP,
				Settings.getI18n().tr("PeriodChooserInput.oneYearTooltip")); //$NON-NLS-1$
		inputs.add(radioInput);

		radioInput = new RadioInput(GROUP_ID, PERIOD_2J);
		radioInput.setName(Settings.getI18n().tr(
				"PeriodChooserInput.twoYearName")); //$NON-NLS-1$
		radioInput.setData(TOOLTIP,
				Settings.getI18n().tr("PeriodChooserInput.twoYearTooltip")); //$NON-NLS-1$
		inputs.add(radioInput);

		radioInput = new RadioInput(GROUP_ID, PERIOD_5J);
		radioInput.setName(Settings.getI18n().tr(
				"PeriodChooserInput.fiveYearName")); //$NON-NLS-1$
		radioInput.setData(TOOLTIP,
				Settings.getI18n().tr("PeriodChooserInput.fiveYearTooltip")); //$NON-NLS-1$
		inputs.add(radioInput);

	}

	/**
	 * @see Input#getValue()
	 */
	@Override
	public List<Object> getValue() {
		// Wir liefern eine Liste aller Ergebnisse zurueck
		final List<Object> values = new ArrayList<Object>();
		for (final Input input : this.inputs) {
			values.add(input.getValue());
		}
		return values;
	}

	/**
	 * @see Input#setValue(Object)
	 */
	@Override
	public void setValue(final Object value) {
		// Hier machen wir nix...
	}

	/**
	 * @see Input#hasChanged()
	 */
	@Override
	public boolean hasChanged() {
		boolean hasChanged = false;

		for (final Input input : this.inputs) {
			if (input.hasChanged()) {
				hasChanged = true;
				break;
			}
		}
		return hasChanged;
	}

	/**
	 * @see Input#getControl()
	 */
	@Override
	public Control getControl() {
		return this.composite;
	}

	/**
	 * @see Input#addListener(Listener)
	 */
	@Override
	public void addListener(final Listener listener) {
		for (final Input input : this.inputs) {
			input.addListener(listener);
		}
	}

	/**
	 * @see Input#setComment(String)
	 */
	@Override
	public void setComment(final String comment) {
		this.comment = comment;
	}

	/**
	 * @see Input#paint(Composite)
	 */
	@Override
	public void paint(final Composite parent) {
		this.paint(parent, 250);

	}

	/**
	 * @see Input#paint(Composite, int)
	 */
	@Override
	public void paint(final Composite parent, final int width) {

		final Font small = Font.SMALL;

		int size = this.inputs.size();
		size++;
		/*
		 * for (Input i : this.inputs) { String name = i.getName(); if (name !=
		 * null) { // size++; } }
		 */

		this.composite = new Composite(parent, SWT.NONE);
		final GridLayout layout = new GridLayout(size, false);
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		layout.horizontalSpacing = 2;
		layout.verticalSpacing = 0;
		this.composite.setLayout(layout);
		final GridData gridDatad = new GridData(GridData.FILL_HORIZONTAL);
		gridDatad.widthHint = width;
		this.composite.setLayoutData(gridDatad);

		for (final Input input : this.inputs) {

			// Checken, ob wir einen Breiten-Hint haben
			final Number weightHint = (Number) input.getData(DATA_WEIGHT);
			final int len = width / size + 0;
			if (weightHint == null) {
				input.paint(this.composite, len);
			} else {
				input.paint(this.composite,
						(int) (len * weightHint.doubleValue()));
			}
			input.getControl()
					.setToolTipText(
							Settings.getI18n()
									.tr("PeriodChooserInput.show", (String) input.getData(TOOLTIP))); //$NON-NLS-1$
		}

		// den Kommentar hinten dran fuegen
		if (this.comment == null) {
			// Einfach noch einen leeren Kommentar anhngen, dass sich
			// ausbreiten kann
			commentLabel = GUI.getStyleFactory().createLabel(this.composite,
					SWT.NONE);
			commentLabel.setText(""); //$NON-NLS-1$
			commentLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		} else {
			commentLabel = GUI.getStyleFactory().createLabel(this.composite,
					SWT.NONE);
			commentLabel.setFont(small.getSWTFont());
			commentLabel.setText(this.comment);
			commentLabel.setForeground(Color.COMMENT.getSWTColor());
			commentLabel.setAlignment(SWT.LEFT);
			commentLabel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		}
		RadioInput.select(GROUP_ID, PERIOD_1J);
	}

	/**
	 * @see Input#focus()
	 */
	@Override
	public void focus() {
		if (this.inputs.size() > 0) {
			this.inputs.get(0).focus();
		}
	}

	/**
	 * @see Input#disable()
	 */
	@Override
	public void disable() {
		this.setEnabled(false);
	}

	/**
	 * @see Input#enable()
	 */
	@Override
	public void enable() {
		this.setEnabled(true);
	}

	/**
	 * @see Input#setEnabled(boolean)
	 */
	@Override
	public void setEnabled(final boolean enabled) {
		for (final Input input : this.inputs) {
			input.setEnabled(enabled);
		}
	}

	/**
	 * @see Input#isEnabled()
	 */
	@Override
	public boolean isEnabled() {

		boolean isenabled = true;
		for (final Input input : this.inputs) {
			if (!input.isEnabled()) {
				isenabled = false;
				break;
			}
		}
		return isenabled;
	}

	/**
	 * @see Input#setMandatory(boolean)
	 */
	@Override
	public void setMandatory(final boolean mandatory) {
		for (final Input input : this.inputs) {
			input.setMandatory(mandatory);
		}
	}

	/**
	 * @see Input#isMandatory()
	 */
	@Override
	public boolean isMandatory() {
		boolean isMandatory = false;
		for (final Input input : this.inputs) {
			if (input.isMandatory()) {
				isMandatory = true;
				break;
			}
		}
		return isMandatory;
	}

	/**
	 * @see Input#setName(String)
	 */
	@Override
	public void setName(final String name) {
		this.name = name;

	}

	/**
	 * @see Input#getName()
	 */
	@Override
	public String getName() {
		return this.name;
	}

	/**
	 * @see Input#setData(String, Object)
	 */
	@Override
	public void setData(final String key, final Object data) {
		this.data.put(key, data);

	}

	/**
	 * @see Input#getData(String)
	 */
	@Override
	public Object getData(final String key) {
		return this.data.get(key);
	}

	/**
	 * Setzt den PeriodChooser auf die angegebene Periode
	 * 
	 * @param period
	 *            auf die derChooser gesetzt werden soll.
	 */
	public void setPeriod(final int period) {
		if (period == PERIOD_1M || period == PERIOD_3M || period == PERIOD_6M
				|| period == PERIOD_1J || period == PERIOD_2J
				|| period == PERIOD_5J) {
			RadioInput.select(GROUP_ID, period);
		}
	}

}
