/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.gui.parts.column;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.OrderHistory;
import de.frankmuenster.jameica.depot.gui.utils.FormatterUtils;
import de.willuhn.jameica.gui.formatter.Formatter;
import de.willuhn.jameica.gui.parts.Column;
import de.willuhn.logging.Logger;

/**
 * Tabellenspalte f&uuml;r die Order-Tabelle mit den Betr&auml;gen in der
 * Standardw&auml;hrung.
 * 
 * @author munstfr
 * @version 1.0
 */
public class OrderTotalInStandartWaehrung extends Column {

	/**
	 * Serial UID
	 */
	private static final long serialVersionUID = -3910333581193391135L;

	/**
	 * Standardkonstruktor
	 */
	public OrderTotalInStandartWaehrung() {
		this(null);
	}

	/**
	 * Konstruktor mit Angabe eines Formatters
	 * 
	 * @param formatter
	 */
	public OrderTotalInStandartWaehrung(final Formatter formatter) {
		super("id", StringUtils.EMPTY, formatter, false, ALIGN_RIGHT); //$NON-NLS-1$
		setName(Settings.getI18n().tr("OrderTotalInStandartWaehrung.totalIn") + Settings.getDepotWaehrung()); //$NON-NLS-1$
	}

	/**
	 * @see de.willuhn.jameica.gui.parts.Column#getFormattedValue(java.lang.Object,
	 *      java.lang.Object)
	 */
	@Override
	public String getFormattedValue(final Object objectId, final Object context) {

		final OrderHistory orderHistory = (OrderHistory) context;
		double betrag;
		String display = StringUtils.EMPTY;
		try {
			betrag = orderHistory.getOrderAmount()
					* orderHistory.getOrderPriceInMainCurrency();

			// Formatter definiert?
			if (this.getFormatter() == null) {
				display = FormatterUtils.CURRENCY_FORMAT.get().format(betrag)
						+ Settings.getDepotWaehrung();
			} else {
				display = this.getFormatter().format(betrag);
			}
		} catch (RemoteException e) {
			Logger.error(
					Settings.getI18n().tr(
							"OrderTotalInStandartWaehrung.formatError", //$NON-NLS-1$
							(String) objectId, (String) context), e);
		}

		return display;
	}

}
