/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.parts.grafiken.analysegrafik;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;
import java.net.URL;
import java.rmi.RemoteException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.time.DateUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.jfree.chart.ChartMouseEvent;
import org.jfree.chart.ChartMouseListener;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.annotations.XYTextAnnotation;
import org.jfree.chart.annotations.XYTitleAnnotation;
import org.jfree.chart.axis.AxisLocation;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.block.BlockBorder;
import org.jfree.chart.entity.ChartEntity;
import org.jfree.chart.entity.EntityCollection;
import org.jfree.chart.entity.LegendItemEntity;
import org.jfree.chart.plot.CombinedDomainXYPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.PlotRenderingInfo;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.CandlestickRenderer;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.chart.renderer.xy.XYAreaRenderer;
import org.jfree.chart.title.LegendTitle;
import org.jfree.data.general.DatasetUtilities;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.experimental.chart.swt.ChartComposite;
import org.jfree.ui.Align;
import org.jfree.ui.RectangleAnchor;
import org.jfree.ui.RectangleEdge;
import org.jfree.ui.RectangleInsets;
import org.jfree.ui.TextAnchor;

import de.frankmuenster.jameica.depot.JaDepot;
import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.AbstractGrafik;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.KursDaten;
import de.frankmuenster.jameica.depot.gui.utils.ColorUtils;
import de.frankmuenster.jameica.depot.kursanalyse.AnalyseNotInListeException;
import de.frankmuenster.jameica.depot.kursanalyse.Analysen;
import de.frankmuenster.jameica.depot.kursanalyse.IKursAnalyse;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseLineStyle;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseTyp;
import de.frankmuenster.jameica.depot.kursanalyse.MethodeAddRemoveListener;
import de.willuhn.jameica.gui.Part;
import de.willuhn.logging.Logger;

/**
 * Klasse die eine Analysegrafik im JFreeChart format bereit stellt. Dabei
 * werden die Analysen dargestellt, die in der Klasse {@link Analysen} enthalten
 * sind.
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
@SuppressWarnings({ "PMD.ExcessiveImports", "PMD.GodClass" })
public class AnalyseGrafik extends AbstractGrafik implements Part, ChartMouseListener {

	/**
	 * Das Schlieen Image
	 */
	private final static String CLOSE_IMAGE_NAME = "/img/black-cross.png"; //$NON-NLS-1$

	/**
	 * Das Parent Composite
	 */
	private transient Composite parent;

	/**
	 * Das ChartComposite
	 */
	private transient ChartComposite frame;

	/**
	 * Anzahl AnalyseCharts
	 */
	private transient int anzAnalyseCharts;

	/**
	 * Die Listener f&uuml; Methoden Hinzug&uuml;gen/Entfernen Aktionen
	 */
	private final transient MethodeAddRemoveListener listener;

	/**
	 * Der Schlieen Button
	 */
	private transient Image img;

	/**
	 * Die Kurswert-Annotion
	 */
	private transient XYTextAnnotation kursWertAnno;

	/**
	 * @param listener
	 */
	public AnalyseGrafik(final MethodeAddRemoveListener listener) {
		super();
		this.listener = listener;
	}

	/**
	 * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void paint(final Composite parent) throws RemoteException {

		this.parent = parent;

		GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
		final CombinedDomainXYPlot plot = new CombinedDomainXYPlot(
				new DateAxis(Settings.getI18n().tr("AnalyseGrafik.axisDate"))); //$NON-NLS-1$
		plot.setGap(10.0);

		// ((DateAxis) plot.getDomainAxis()).setTimeline(SegmentedTimeline
		// .newMondayThroughFridayTimeline());

		// return a new chart containing the overlaid plot...
		chart = new JFreeChart(title, JFreeChart.DEFAULT_TITLE_FONT, plot, true);
		chart.removeLegend();

		// frame = new ChartComposite(parent, SWT.NONE, chart, false, false,
		// false,false, false);
		frame = new ChartComposite(this.parent, SWT.NONE, chart, false);

		frame.addChartMouseListener(this);

		gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
		gridData.heightHint = 300 + (anzAnalyseCharts * 200);
		frame.setLayout(new FillLayout());
		frame.setLayoutData(gridData);

		frame.pack();
		frame.setVisible(true);

		frame.setDomainZoomable(false);
		frame.setRangeZoomable(false);

	}

	/**
	 * Zeichnet die Grafik neu
	 */
	@Override
	public void reload() {

		if (frame != null) {

			frame.setVisible(false);
			CombinedDomainXYPlot mainPlot = (CombinedDomainXYPlot) chart.getPlot();

			// Entferne all Plots
			while (mainPlot.getSubplots().size() > 0) {
				final XYPlot plot = (XYPlot) mainPlot.getSubplots().get(0);
				mainPlot.remove(plot);

			}

			frame.dispose();

			mainPlot = new CombinedDomainXYPlot(new DateAxis(Settings.getI18n().tr("AnalyseGrafik.axisDate"))); //$NON-NLS-1$
			mainPlot.setGap(10.0);

			// return a new chart containing the overlaid plot...
			chart = new JFreeChart(title, JFreeChart.DEFAULT_TITLE_FONT, mainPlot, false);

			frame = new ChartComposite(this.parent, SWT.NONE, chart, false);
			frame.setVisible(false);
			frame.addChartMouseListener(this);

			frame.setDomainZoomable(false);
			frame.setRangeZoomable(false);

			// Erzeuge die Plots
			createCombinedChart(mainPlot);

			final GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
			if (anzAnalyseCharts == 0) {
				gridData.heightHint = 350;
			} else {
				gridData.heightHint = 200 + (anzAnalyseCharts * 150);
			}

			// Datumanzeige anpassen
			final DateAxis dateAxis = ((DateAxis) mainPlot.getDomainAxis());
			final Date min = dateAxis.getMinimumDate();
			final Date max = dateAxis.getMaximumDate();
			final long diff = max.getTime() - min.getTime();
			final long oneYear = 1000l * 60 * 60 * 24 * 365;

			SimpleDateFormat dateFormat;
			if (diff > oneYear) {
				dateFormat = new SimpleDateFormat("MM-yy", Locale.GERMAN); //$NON-NLS-1$
				dateAxis.setLabel(Settings.getI18n().tr("AnalyseGrafik.lbldateAxisMonthYear")); //$NON-NLS-1$
			} else {
				dateFormat = new SimpleDateFormat("dd.MM", Locale.GERMAN); //$NON-NLS-1$
				final int yearMin = de.frankmuenster.jameica.depot.gui.utils.DateUtils.getYear(min);
				final int yearMax = de.frankmuenster.jameica.depot.gui.utils.DateUtils.getYear(max);
				if (yearMin == yearMax) {
					dateAxis.setLabel(Settings.getI18n().tr("AnalyseGrafik.lbldateAxisDayInMonthOfYear", //$NON-NLS-1$
							Integer.toString(yearMin)));
				} else {
					dateAxis.setLabel(Settings.getI18n().tr("AnalyseGrafik.lbldateAxisDayInMonthOfYears", //$NON-NLS-1$
							Integer.toString(yearMin), Integer.toString(yearMax)));
				}
			}
			dateAxis.setDateFormatOverride(dateFormat);

			frame.setLayout(new FillLayout());
			frame.setLayoutData(gridData);

			frame.update();
			frame.pack();

			frame.getParent().pack();
			frame.getParent().update();
			frame.getParent().getParent().pack();
			frame.getParent().getParent().update();

			frame.setVisible(true);
		}

	}

	private CombinedDomainXYPlot createCombinedChart(final CombinedDomainXYPlot mainPlot) {

		anzAnalyseCharts = 0;
		int anzKursPlotLines = 0;
		final List<XYPlot> analysePlots = new ArrayList<XYPlot>();

		final DateAxis dateAxisKurs = new DateAxis(Settings.getI18n().tr("AnalyseGrafik.axisDate")); //$NON-NLS-1$
		final NumberAxis rangeAxisKurs = new NumberAxis(Settings.getI18n().tr("AnalyseGrafik.axisValue")); //$NON-NLS-1$
		dateAxisKurs.setLabelFont(FONT_AXIS);
		rangeAxisKurs.setLabelFont(FONT_AXIS);
		rangeAxisKurs.setAutoRangeIncludesZero(false);

		final XYPlot subplotKurs = new XYPlot(null, dateAxisKurs, rangeAxisKurs, null);

		subplotKurs.setRangeAxisLocation(AxisLocation.BOTTOM_OR_LEFT);

		subplotKurs.setDomainCrosshairVisible(true);
		subplotKurs.setDomainCrosshairLockedOnData(false);

		for (final IKursAnalyse<?, ?> iKursAnalyse : Analysen.getInstance()) {

			if (iKursAnalyse.getType().equals(KursAnalyseTyp.OVERLAY_LINE) && !isShowDax()) {
				addAnalyseToPLot(iKursAnalyse, subplotKurs, anzKursPlotLines);
				anzKursPlotLines += iKursAnalyse.getLineCount();
			} else if (iKursAnalyse.getType().equals(KursAnalyseTyp.OVERLAY_BAR)) {
				Logger.warn(Settings.getI18n().tr("AnalyseGrafik.overlayNotImplement")); //$NON-NLS-1$
			} else if (iKursAnalyse.getType().equals(KursAnalyseTyp.SEPERATE_GRAPH)) {
				final XYPlot analysePlot = createSeperateAnaylsePlot(iKursAnalyse);
				analysePlots.add(analysePlot);
			}
		}

		// Kurschart Dataset und Renderer setzten abhngig vom Charttyp
		setKursChartDatasetAndRenderer(subplotKurs, anzKursPlotLines);

		// Legende hinzufgen
		final LegendTitle legendTitle = new LegendTitle(subplotKurs);
		legendTitle.setItemFont(new Font("Dialog", Font.PLAIN, 9)); //$NON-NLS-1$
		legendTitle.setBackgroundPaint(new Color(200, 200, 255, 100));
		legendTitle.setFrame(new BlockBorder(Color.white));
		legendTitle.setPosition(RectangleEdge.BOTTOM);
		final RectangleInsets padding = new RectangleInsets(4, 2, 0, 10);
		legendTitle.setItemLabelPadding(padding);
		final XYTitleAnnotation titleAnnotation = new XYTitleAnnotation(1.0, 0.00, legendTitle,
				RectangleAnchor.BOTTOM_RIGHT);

		titleAnnotation.setMaxWidth(0.48);
		subplotKurs.addAnnotation(titleAnnotation);

		// Der Kurs-Chart-Plot in dem Hauptplot setzten
		mainPlot.add(subplotKurs, 4);

		// Und nun die Analyseplots drunter
		for (final XYPlot xyPlot : analysePlots) {
			xyPlot.setDomainCrosshairVisible(true);
			xyPlot.setDomainCrosshairLockedOnData(false);
			mainPlot.add(xyPlot, 3);
		}
		mainPlot.setOrientation(PlotOrientation.VERTICAL);

		return mainPlot;

	}

	/**
	 * Kurschart Dataset und Renderer setzten abhngig vom Charttyp.
	 * 
	 * @param subplotKurs
	 *            der Kurs-Chart-Plot
	 * @param anzKursPlotLines
	 *            Anzahl vorhandener Plotlinien
	 */
	private void setKursChartDatasetAndRenderer(final XYPlot subplotKurs, final int anzKursPlotLines) {

		if (showCandle) {

			final CandlestickRenderer rendererCandle = new CandlestickRenderer();
			rendererCandle.setDrawVolume(true);
			rendererCandle.setSeriesPaint(0, Color.lightGray);
			rendererCandle.setSeriesVisibleInLegend(0, false);
			rendererCandle.setBaseToolTipGenerator(null);

			subplotKurs.setDataset(anzKursPlotLines, getDatasetCandleStick());
			subplotKurs.setRenderer(anzKursPlotLines, rendererCandle);

		} else if (showDax) {
			final StandardXYItemRenderer rendererKurs = new StandardXYItemRenderer();
			final StandardXYItemRenderer rendererDax = new StandardXYItemRenderer();
			rendererKurs.setSeriesPaint(0, Color.GREEN);
			rendererDax.setSeriesPaint(1, Color.BLUE);

			final TimeSeriesCollection seriesKurs = new TimeSeriesCollection();
			final TimeSeriesCollection seriesDax = new TimeSeriesCollection();
			seriesKurs.addSeries(getDatasetKursDaxVergleich().getSeries(0));
			seriesDax.addSeries(getDatasetKursDaxVergleich().getSeries(1));

			subplotKurs.setDataset(anzKursPlotLines, seriesKurs);
			subplotKurs.setRenderer(anzKursPlotLines, rendererKurs);
			subplotKurs.setDataset(anzKursPlotLines + 1, seriesDax);
			subplotKurs.setRenderer(anzKursPlotLines + 1, rendererDax);

			subplotKurs.getRangeAxis().setLabel(Settings.getI18n().tr("AnalyseGrafik.procentageAxisLabel")); //$NON-NLS-1$

		} else {
			final XYAreaRenderer rendererKurs = new XYAreaRenderer();
			rendererKurs.setSeriesPaint(0, Color.lightGray);
			rendererKurs.setSeriesFillPaint(0, Color.lightGray);
			rendererKurs.setSeriesOutlinePaint(0, Color.darkGray);
			rendererKurs.setOutline(true);
			rendererKurs.setSeriesVisibleInLegend(0, false);

			subplotKurs.setDataset(anzKursPlotLines, getDataSetKurs());
			subplotKurs.setRenderer(anzKursPlotLines, rendererKurs);
		}
	}

	/**
	 * F&uuml;gt eine Analyse in den Kurs-Chart-Plot ein.
	 * 
	 * @param iKursAnalyse
	 *            die Kursanalyse
	 * @param subplotKurs
	 *            der Kurs-Chart-Plot
	 * @param anzKursPlotLines
	 *            die Nummer der Linie an
	 */
	@SuppressWarnings("PMD.AvoidInstantiatingObjectsInLoops")
	private void addAnalyseToPLot(final IKursAnalyse<?, ?> iKursAnalyse, final XYPlot subplotKurs,
			final int anzKursPlotLines) {

		final BasicStroke dotted = new BasicStroke(2.0f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 1.0f,
				new float[] { 1.0f, 2.0f }, 0.0f);

		Logger.debug("Adding OVERLAY_LINE " + iKursAnalyse.getLegendTitel()); //$NON-NLS-1$

		final KursDaten[] kursDaten = (KursDaten[]) iKursAnalyse.getResult(fromDate, toDate);
		for (int i = 0; i < kursDaten.length; i++) {
			// Datenwerte
			final KursDaten analyseDaten = kursDaten[i];
			final TimeSeries timeSeries = analyseDaten.getTimeSeriesFromKursDaten(iKursAnalyse.getLegendTitel());
			final TimeSeriesCollection series = new TimeSeriesCollection();
			series.addSeries(timeSeries);

			// Renderer
			final StandardXYItemRenderer rendererLine = new StandardXYItemRenderer();
			rendererLine.setSeriesPaint(0, ColorUtils.toAwtColor(iKursAnalyse.getColor()[i]));
			if (i > 0) {
				rendererLine.setSeriesVisibleInLegend(0, false);
			}
			// LineStyle
			if (iKursAnalyse.getLineStyle() != null) {
				final KursAnalyseLineStyle[] lineStyles = iKursAnalyse.getLineStyle();
				if (lineStyles[i].equals(KursAnalyseLineStyle.DOTTED)) {
					rendererLine.setSeriesStroke(0, dotted);
				}
			}

			// Linie hinzufgen
			subplotKurs.setDataset(anzKursPlotLines + i, series);
			subplotKurs.setRenderer(anzKursPlotLines + i, rendererLine);

		}
	}

	private XYPlot createSeperateAnaylsePlot(final IKursAnalyse<?, ?> iKursAnalyse) {

		Logger.debug("Adding SEPERATE_GRAPH " + iKursAnalyse.getLegendTitel()); //$NON-NLS-1$

		anzAnalyseCharts++;
		final XYPlot analysePlot = (XYPlot) iKursAnalyse.getResult(fromDate, toDate);

		analysePlot.getRangeAxis().setLabelFont(FONT_AXIS);
		if (analysePlot.getDomainAxis() != null) {
			analysePlot.getDomainAxis().setLabelFont(FONT_AXIS);
		}

		if (img == null) {
			// schlieen Knopf
			final URL imgURL = JaDepot.class.getResource(CLOSE_IMAGE_NAME);
			final Toolkit toolkit = Toolkit.getDefaultToolkit();
			img = toolkit.getImage(imgURL);

			// Mit Mediatracker warten, bis das Bild geladen ist.
			final Component component = new Component() {
				private static final long serialVersionUID = 1L;
			};
			final MediaTracker tracker = new MediaTracker(component);
			tracker.addImage(img, 0);
			try {
				tracker.waitForAll();
			} catch (InterruptedException ie) {
			}

		}
		// ...und das Bild nun malen
		analysePlot.setBackgroundImage(img);
		analysePlot.setBackgroundImageAlignment(Align.TOP_RIGHT);

		return analysePlot;
	}

	/**
	 * handle mouse click on Chart
	 */
	@Override
	public void chartMouseClicked(final ChartMouseEvent event) {

		if (event.getTrigger().getButton() != MouseEvent.BUTTON1) {
			reload();
			return;
		}

		// Wo wurde geklickt
		final int mouseX = event.getTrigger().getX();
		final int mouseY = event.getTrigger().getY();

		// Das Chart
		final JFreeChart chart = event.getChart();

		// Finde den Plot
		final CombinedDomainXYPlot mainPlot = (CombinedDomainXYPlot) chart.getPlot();

		final int anzSubPlots = mainPlot.getSubplots().size();

		// Welcher Plot wurde angeklickt?
		int clickedPlot = -1;
		Rectangle2D clickedPlotArea = new Rectangle2D.Double();
		for (int i = 0; i < anzSubPlots; i++) {

			final Rectangle2D plotArea = frame.getChartRenderingInfo().getPlotInfo().getSubplotInfo(i).getDataArea();
			Logger.warn(plotArea.toString());
			if (plotArea.contains(mouseX, mouseY)) {
				Logger.debug("Plot " + i + " angeklickt"); //$NON-NLS-1$ //$NON-NLS-2$
				clickedPlot = i;
				clickedPlotArea = plotArea;
				break;
			}
		}

		if (clickedPlot == 0) {
			removeAnalysisFromMainPlot(mouseX, mouseY);
		} else if (clickedPlot > 0) {
			removeAdditionalAnalysisPlot(mainPlot, clickedPlot, clickedPlotArea, mouseX, mouseY);
		} else {
			Logger.debug("Kein Plot geklicked"); //$NON-NLS-1$
		}

		XYPlot plot1 = ((XYPlot) ((CombinedDomainXYPlot) this.chart.getPlot()).getSubplots().get(0));
		final DateAxis dateAxis = (DateAxis) plot1.getDomainAxis();
		final PlotRenderingInfo plotInfo = frame.getChartRenderingInfo().getPlotInfo();
		final double val = dateAxis.java2DToValue(mouseX, plotInfo.getDataArea(), plot1.getDomainAxisEdge());
		final Date xDate = new Date(new Double(val).longValue());
		Logger.debug("val: " + val + "xdate: " + xDate.toString()); //$NON-NLS-1$ //$NON-NLS-2$
		DateUtils.setHours(xDate, 0);
		DateUtils.setMinutes(xDate, 0);
		DateUtils.setSeconds(xDate, 0);
		DateUtils.setMilliseconds(xDate, 0);
		reload();

		// Crosshair wieder malen
		plot1 = ((XYPlot) ((CombinedDomainXYPlot) this.chart.getPlot()).getSubplots().get(0));
		plot1.setDomainCrosshairVisible(true);
		plot1.setDomainCrosshairValue(xDate.getTime(), true);
		// Y-Wert and CrossHier X Position ermitteln.
		final double kursWert = DatasetUtilities.findYValue(plot1.getDataset(), 0, xDate.getTime() - (60000));
		createKursPlotAnnotion(plot1, xDate.getTime(), kursWert);

	}

	/**
	 * Entfernt ein Analyse aus dem Main-Plot
	 * 
	 * @param mouseX
	 * @param mouseY
	 */
	private void removeAnalysisFromMainPlot(final int mouseX, final int mouseY) {

		// leider muessen wir ueber alle Entities iterieren
		final EntityCollection entities = frame.getChartRenderingInfo().getEntityCollection();

		@SuppressWarnings("unchecked")
		final Iterator<ChartEntity> chartEntities = entities.iterator();

		while (chartEntities.hasNext()) {

			final ChartEntity chartEntity = chartEntities.next();

			// Wir suchen nur die LegenItemEntitys
			if (chartEntity instanceof LegendItemEntity) {

				final LegendItemEntity lie = (LegendItemEntity) chartEntity;
				final Rectangle2D legendItemArea = lie.getArea().getBounds2D();

				if (legendItemArea.contains(mouseX, mouseY)) {
					final String analyseTitel = lie.getSeriesKey().toString();
					Logger.info(Settings.getI18n().tr("AnalyseGrafik.removeAnalysis") + analyseTitel); //$NON-NLS-1$
					if (Analysen.getInstance().conatinsAnalyseWithLegendTitel(analyseTitel)) {
						try {
							final IKursAnalyse<?, ?> ana = Analysen.getInstance()
									.getAnalyseWithLegendTitel(analyseTitel);
							listener.notifyRemoveMethode(ana);
						} catch (AnalyseNotInListeException e) {
							Logger.debug("Analysis not in list"); //$NON-NLS-1$
						}
					}

				}
			}
		}

	}

	/**
	 * Entfernt eine kompletten AnalysePlot
	 * 
	 * @param mainPlot
	 *            der Haupt-Plot mit allen Grafiken
	 * @param clickedPlot
	 *            die Sub-Plot Nummer
	 * @param clickedPlotArea
	 *            das Rectangle2D des Sub-Plots
	 * @param mouseX
	 *            Maus X
	 * @param mouseY
	 *            Maus Y
	 */
	private void removeAdditionalAnalysisPlot(final CombinedDomainXYPlot mainPlot, final int clickedPlot,
			final Rectangle2D clickedPlotArea, final int mouseX, final int mouseY) {

		// hole Plot und teste ob es ein XYPlot ist
		final Object plot = mainPlot.getSubplots().get(clickedPlot);
		if (plot instanceof XYPlot) {
			// Rectangle der CloseBox berechnen
			final double closeBoxX = clickedPlotArea.getBounds2D().getX() + clickedPlotArea.getBounds2D().getWidth()
					- 20.0;
			final double closeBoxY = clickedPlotArea.getBounds2D().getY();
			final Rectangle2D closeBox = new Rectangle2D.Double(closeBoxX, closeBoxY, 20, 20);

			// Test ob close-Button geklickt wurde
			if (closeBox.contains(mouseX, mouseY)) {
				// hole PlotKey
				final String plotKey = ((XYPlot) plot).getDataset(0).getSeriesKey(0).toString();

				Logger.info(Settings.getI18n().tr("AnalyseGrafik.removeAnalysis") + plotKey); //$NON-NLS-1$
				// Remove Analyse, falls vorhanden
				if (Analysen.getInstance().conatinsAnalyseWithLegendTitel(plotKey)) {
					try {
						final IKursAnalyse<?, ?> ana = Analysen.getInstance().getAnalyseWithLegendTitel(plotKey);
						listener.notifyRemoveMethode(ana);
					} catch (AnalyseNotInListeException e) {
						Logger.debug("Analysis not in list"); //$NON-NLS-1$
					}
				}
			}
		}

	}

	/**
	 * Aktualisiert das Crosshair &uuml;ber alle Grafiken (senkrechte Linie)
	 */
	@Override
	public void chartMouseMoved(final ChartMouseEvent event) {

		final double crossHairX = ((XYPlot) ((CombinedDomainXYPlot) this.chart.getPlot()).getSubplots().get(0))
				.getDomainCrosshairValue();

		// Wo wurde geklickt
		final int mouseX = event.getTrigger().getX();
		// int mouseY = event.getTrigger().getY();

		Logger.debug("X:" + mouseX + ", xx: " + (long) crossHairX); //$NON-NLS-1$ //$NON-NLS-2$
		Logger.debug("Time: " + new Date((long) crossHairX).toString()); //$NON-NLS-1$

		final XYPlot kursPlot = ((XYPlot) ((CombinedDomainXYPlot) this.chart.getPlot()).getSubplots().get(0));
		final DateAxis dateAxis = (DateAxis) kursPlot.getDomainAxis();

		// ValueAxis valueAxis = kursPlot.getRangeAxis();

		final Date minDate = dateAxis.getMinimumDate();
		final Date maxDate = dateAxis.getMaximumDate();

		final PlotRenderingInfo plotInfo = frame.getChartRenderingInfo().getPlotInfo();

		final int minX = (int) dateAxis.dateToJava2D(minDate, plotInfo.getDataArea(), kursPlot.getDomainAxisEdge());
		final int maxX = (int) dateAxis.dateToJava2D(maxDate, plotInfo.getDataArea(), kursPlot.getDomainAxisEdge());

		Logger.debug("xMinDate: " + minX + ", xMaxDate: " + maxX); //$NON-NLS-1$ //$NON-NLS-2$

		final double val = dateAxis.java2DToValue(mouseX, plotInfo.getDataArea(), kursPlot.getDomainAxisEdge());
		final Date xDate = new Date(new Double(val).longValue());

		DateUtils.setHours(xDate, 0);
		DateUtils.setMinutes(xDate, 0);
		DateUtils.setSeconds(xDate, 0);
		DateUtils.setMilliseconds(xDate, 0);

		// Setzte Crosshair in kursPlot
		kursPlot.setDomainCrosshairValue(xDate.getTime(), true);

		// Y-Wert and CrossHier X Position ermitteln.
		final double kursWert = DatasetUtilities.findYValue(kursPlot.getDataset(), 0, xDate.getTime() - (60000));
		Logger.debug("val: " + val + "val1: " + kursWert + "xdate: " //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				+ xDate.toString());

		createKursPlotAnnotion(kursPlot, xDate.getTime(), kursWert);

		// Und nun in allen anderen Plots
		@SuppressWarnings("unchecked")
		final List<XYPlot> subPlots = ((CombinedDomainXYPlot) this.chart.getPlot()).getSubplots();
		for (final XYPlot p : subPlots) {
			p.setDomainCrosshairValue(xDate.getTime(), true);
		}

	}

	/**
	 * Den Kurswert als Annotation ausgegeben.
	 * 
	 * @param kursPlot
	 *            Plot
	 * @param mouseX
	 *            X-Position
	 * @param kursWert
	 *            der Kurswert
	 */
	private void createKursPlotAnnotion(final XYPlot kursPlot, final double mouseX, final double kursWert) {

		if (kursWertAnno != null) {
			kursPlot.removeAnnotation(kursWertAnno);
		}

		if (!showCandle) {
			final StringBuffer annotationValue = new StringBuffer(DECIMAL_FOMAT.format(kursWert));
			if (showDax) {
				annotationValue.append(" %"); //$NON-NLS-1$
			} else {
				annotationValue.append(" EUR"); //$NON-NLS-1$
			}
			kursWertAnno = new XYTextAnnotation(annotationValue.toString(), mouseX + 5, kursWert);
			// annotation.setOutlineVisible(true);
			kursWertAnno.setFont(new Font("SansSerif", Font.BOLD, 12)); //$NON-NLS-1$
			kursWertAnno.setPaint(Color.blue);
			kursWertAnno.setTextAnchor(TextAnchor.BOTTOM_LEFT);
			kursPlot.addAnnotation(kursWertAnno);
		}

	}

}
