/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.utils;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

/**
 * Utiliy Klasse fr Datumsberechnungen
 * 
 * @author munstfr
 * @version 1.0
 */
public final class DateUtils {

	/**
	 * Millisekunden eines Tages
	 */
	private static final double DAY_MILLIS = 24.0 * 60.0 * 60.0 * 1000.0;

	/**
	 * Standardkonstruktor
	 */
	private DateUtils() {
		// absichlich leer
	}

	/**
	 * Addiert die Anzahl an Tagen und liefert ein neues Objekt basierend auf
	 * Now()
	 * 
	 * @param days
	 *            Anzahl die addiert wird (kann negative sein)
	 * @return das new Date Objekt mit der Anzahl addiert
	 */
	public static Date addDays(final int days) {
		final Calendar cal = Calendar.getInstance();
		cal.add(Calendar.DATE, days);
		return cal.getTime();
	}

	/**
	 * Subtrahiert die Anzahl an Tagen und liefert ein neues Objekt basierend
	 * auf Now()
	 * 
	 * @param days
	 *            Anzahl die addiert wird (kann negative sein)
	 * @return das new Date Objekt mit der Anzahl subtrahiert
	 */
	public static Date subtractDays(final int days) {
		final Calendar cal = Calendar.getInstance();
		cal.add(Calendar.DATE, -days);
		return cal.getTime();
	}

	/**
	 * Berechnet die Differenz zwischen Jetzt und dem Date-Objekt
	 * 
	 * @param date
	 *            das Date-Objekt, nicht Null
	 * @return die Differenz in Millisekunden
	 */
	public static long dateDiff(final Date date) {
		return dateDiff(date, new Date());
	}

	/**
	 * Berechnet die Differenz zwischen den beiden Date-Objekt
	 * 
	 * @param fromDate
	 *            das Anfangsdatum Date-Objekt, nicht Null
	 * @param toDate
	 *            das Enddatum Date-Objekt, nicht Null
	 * @return die Differenz in Millisekunden
	 */
	public static long dateDiff(final Date fromDate, final Date toDate) {

		return (long) ((toDate.getTime() - fromDate.getTime()) / DAY_MILLIS);

	}
	
	/**
	 * Liefert das Jahr des Datums als Integerwert.
	 * 
	 * @param date das Datum
	 * @return das Jahr als Integerwert
	 */
	public static int getYear(final Date date) {
		final Calendar calendar = new GregorianCalendar();
		calendar.setTime(date);
		return calendar.get(Calendar.YEAR);
	}

	/**
	 * Liefert den Monat des Datums als Integerwert. Januar gleich 1, Dezember gleich 12.
	 * 
	 * @param date das Datum
	 * @return den Monat als Integerwert 
	 */
	public static int getMonth(final Date date) {
		final Calendar calendar = new GregorianCalendar();
		calendar.setTime(date);
		return calendar.get(Calendar.MONTH + 1);
	}

	/**
	 * Liefert den Tag des Datums als Integerwert.
	 * 
	 * @param date das Datum
	 * @return den Tag als Integerwert
	 */
	public static int getDay(final Date date) {
		final Calendar calendar = new GregorianCalendar();
		calendar.setTime(date);
		return calendar.get(Calendar.DAY_OF_MONTH);
	}

}
