/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.views;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.gui.action.OrderDetailAction;
import de.frankmuenster.jameica.depot.gui.action.WertpapierAktualisiernAction;
import de.frankmuenster.jameica.depot.gui.controller.DepotController;
import de.willuhn.jameica.gui.AbstractView;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Klasse die die View zur Depotbersicht bereit stellt.
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class DepotView extends AbstractView {

	// private View view;
	/**
	 * Der Depot-Kontroller
	 */
	private transient DepotController control;

	/**
	 * Standardkonstruktor
	 */
	public DepotView() {
		super();
	}

	/**
	 * @see de.willuhn.jameica.gui.AbstractView#bind()
	 */
	@Override
	public void bind() throws RemoteException {

		control = new DepotController(this);

		final SimpleContainer left = new SimpleContainer(getParent());

		left.addHeadline(Settings.getI18n().tr("DepotView.hlStockOverview")); //$NON-NLS-1$

		// WertpapierListePart
		left.addPart(control.getAktienListe());
		left.addText(StringUtils.EMPTY, true);

		if (control.getAktienListe().isShowWaehrungsSpalte()) {
			left.addHeadline(Settings.getI18n().tr("DepotView.hlExchangeRates")); //$NON-NLS-1$
			left.addPart(control.getWechselKursInfos());
		}

		if (Settings.isShowHibiscusSummery()) {
			left.addHeadline(Settings.getI18n().tr("DepotView.hlTotal")); //$NON-NLS-1$
			final SimpleContainer spHibiscusSummery = new SimpleContainer(
					left.getComposite(), false, 6);
			spHibiscusSummery.addLabelPair(
					Settings.getI18n().tr("DepotView.lblStockTotal"), //$NON-NLS-1$
					control.getJaDepotTotal());
			spHibiscusSummery.addText(" ", false); //$NON-NLS-1$
			spHibiscusSummery.addText(" ", false); //$NON-NLS-1$
			spHibiscusSummery.addLabelPair(
					Settings.getI18n().tr("DepotView.lblHibiscusTotal"), //$NON-NLS-1$
					control.getHibiscusTotal());
			spHibiscusSummery.addText(" ", false); //$NON-NLS-1$
			spHibiscusSummery.addText(" ", false); //$NON-NLS-1$
			spHibiscusSummery.addLabelPair(
					Settings.getI18n().tr("DepotView.lblTotal"), //$NON-NLS-1$
					control.getHibiscusJaDepotTotal());
		}

		final ButtonArea actionButtons = new ButtonArea();
		final WertpapierAktualisiernAction wpUpdateAction = new WertpapierAktualisiernAction();
		wpUpdateAction.addListener(control);
		actionButtons.addButton(Settings.getI18n().tr("DepotView.btnRefresh"), //$NON-NLS-1$
				wpUpdateAction, null, false, "view-refresh.png"); //$NON-NLS-1$
		actionButtons.addButton(Settings.getI18n().tr("DepotView.btnNewStock"), //$NON-NLS-1$
				new OrderDetailAction(), null, false, "document-new.png"); //$NON-NLS-1$
		left.addButtonArea(actionButtons);

	}

	/**
	 * @see de.willuhn.jameica.gui.AbstractView#unbind()
	 */
	@Override
	public void unbind() throws ApplicationException {

		Logger.debug("DepotView: unbind"); //$NON-NLS-1$

		super.unbind();
		control = null; // NOPMD
	}

}
