/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.kursanalyse;

import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

/**
 * Abstrakte Klassen zum Behandeln von Analysehinzuf&uuml;gen-Events.
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public abstract class AbstractHandleAddAction implements IHandleAddAction {

	/**
	 * Die Listener fr einen AddMethode-Event
	 */
	private transient final Set<MethodeAddRemoveListener> listeners = new CopyOnWriteArraySet<MethodeAddRemoveListener>();

	/**
	 * Standardkonstruktor
	 */
	public AbstractHandleAddAction() {
		// Absichtlich leer
	}

	/**
	 * Fgt eine Listener hinzu
	 * 
	 * @param listener
	 *            der hinzugefgt werden soll
	 */
	@Override
	public final void addListener(final MethodeAddRemoveListener listener) {
		listeners.add(listener);
	}

	/**
	 * Entfernt einen Listener aus der Liste
	 * 
	 * @param listener
	 *            der entfernt werden soll
	 */
	@Override
	public final void removeListener(final MethodeAddRemoveListener listener) {
		listeners.remove(listener);
	}

	/**
	 * Benachrichtigen der registrierten Listeners mit bergabe des gefundenen
	 * Strings
	 * 
	 * @param methode
	 */
	@Override
	public final void notifyListeners(final IKursAnalyse<?, ?> methode) {
		for (final MethodeAddRemoveListener listener : listeners) {
			listener.notifyAddMethode(methode);
		}
	}

}
