/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.kursanalyse.EMA; // NOPMD 

import java.util.Date;

import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.WertpapierHistory;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.DateValuePair;
import de.frankmuenster.jameica.depot.gui.parts.grafiken.KursDaten;
import de.frankmuenster.jameica.depot.kursanalyse.AbstractKursAnalyse;
import de.frankmuenster.jameica.depot.kursanalyse.KursAnalyseParameter;
import de.willuhn.jameica.gui.Action;

/**
 * ExponentialMovingAverage
 * 
 * @author munstfr
 * @version 1.0
 */
public class ExponentialMovingAverage extends
		AbstractKursAnalyse<Integer, KursDaten[]> {

	/**
	 * Methodenname
	 */
	private final static String NAME = "EMA"; //$NON-NLS-1$

	/**
	 * ExponentialMovingAverage Daten
	 */
	private transient KursDaten ema;

	/**
	 * Konstruktor der nurt super() aufruft.
	 */
	public ExponentialMovingAverage() {
		super();
	}

	/**
	 * Konstruktor mit Angabe der Kursdaten und des Parameters.
	 * 
	 * @param wertpapierHistory
	 * @param parameter
	 */
	public ExponentialMovingAverage(final WertpapierHistory wertpapierHistory,
			final KursAnalyseParameter<Integer> parameter) {
		super(wertpapierHistory, parameter);
	}

	/**
	 * Liefert den Methodennamen
	 * 
	 * @return der Methodennamen
	 */
	public static String getMethodName() {
		return NAME;
	}

	/**
	 * Liefert die AddAction
	 * 
	 * @return die EmaHandleAddAction
	 */
	public static Action getHandleAction() {
		return new EmaHandleAddAction();
	}

	/**
	 * @see AbstractKursAnalyse#getLegendTitel ()
	 */
	@Override
	public String getLegendTitel() {
		return "EMA" + parameter.getValue(); //$NON-NLS-1$
	}

	/**
	 * @see AbstractKursAnalyse#getResult(Date, Date)
	 */
	@Override
	public KursDaten[] getResult(final Date fromDate, final Date toDate) {

		//this.parameter = (KursAnalyseParameter<Integer>) parameter;

		if (ema == null) {
			callculate();
		}

		KursDaten result[] = new KursDaten[1];
		final KursDaten emaSeries = new KursDaten();

		for (int i = 0; i < ema.size(); i++) {
			final DateValuePair orgValue = ema.get(i);
			if (fromDate.getTime() < orgValue.getDate().getTime()
					&& orgValue.getDate().getTime() < toDate.getTime()) {
				emaSeries.add(orgValue);

			}
		}
		result[0] = emaSeries;
		return result;

	}

	/**
	 * Hier wird gerechnet
	 */
	private void callculate() {

		// Das Intervall
		final Integer interval = parameter.getValue();

		// EMA Daten
		ema = new KursDaten();

		double expWight;
		expWight = 2.0 / (interval + 1.0);

		ema.add(new DateValuePair(closeDaten.get(0).getDate(), closeDaten
				.get(0).getValue()));
		for (int i = 1; i < closeDaten.size(); i++) {
			final double emaValue = ema.get(i - 1).getValue()
					+ (expWight * (closeDaten.get(i).getValue() - ema
							.get(i - 1).getValue()));
			ema.add(new DateValuePair(closeDaten.get(i).getDate(), emaValue)); // NOPMD
		}
	}
}
