/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.kursanalyse.RSI; //NOPMD

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.util.Date;

import org.jfree.chart.annotations.XYTitleAnnotation;
import org.jfree.chart.axis.AxisLocation;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.SegmentedTimeline;
import org.jfree.chart.block.BlockBorder;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.chart.renderer.xy.XYDifferenceRenderer;
import org.jfree.chart.title.LegendTitle;
import org.jfree.data.time.Day;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.ui.RectangleAnchor;
import org.jfree.ui.RectangleEdge;
import org.jfree.ui.RectangleInsets;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.kursanalyse.AbstractKursAnalyse;
import de.willuhn.jameica.gui.Action;
import de.willuhn.logging.Logger;

/**
 * Momentum Methode
 * 
 * @author munstfr
 * @version 1.0
 */
public class RelativeStrengthIndex extends AbstractKursAnalyse<Integer, XYPlot> {

	/**
	 * Methodenname
	 */
	private final static String NAME = "RSI"; //$NON-NLS-1$

	/**
	 * RSI max. Limit (100)
	 */
	private final static int RSI_ABOVE_100 = 100;

	/**
	 * Daten der RSI Serie
	 */
	private transient TimeSeries serieRsi;

	/**
	 * Daten der RSI Serie unter 30%
	 */
	private transient TimeSeries serie30;

	/**
	 * Daten der RSI Serie ber 70%
	 */
	private transient TimeSeries serie70;

	/**
	 * Renderer der RSI Serie
	 */
	private transient StandardXYItemRenderer rendererRsi;

	/**
	 * Renderer f&uuml; der Bereich &uuml;ber 70%
	 */
	private transient XYDifferenceRenderer diff1;

	/**
	 * Renderer f&uuml; der Bereich unter 30%
	 */
	private transient XYDifferenceRenderer diff2;

	/**
	 * Standardkonstruktor
	 */

	public RelativeStrengthIndex() {
		super();
	}

	/**
	 * Liefert den Methodennamen
	 * 
	 * @return der Methodennamen
	 */
	public static String getMethodName() {
		return NAME;
	}

	/**
	 * Liefert die Add Aktion
	 * 
	 * @return die RelativeStrengthIndexHandleAction
	 */
	public static Action getHandleAction() {
		return new RelativeStrengthIndexHandleAction();
	}

	/**
	 * 
	 * @see AbstractKursAnalyse#getLegendTitel()
	 */
	@Override
	public String getLegendTitel() {
		final int period = parameter.getValue();
		return "RSI (" + period + ")"; //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * 
	 * @see AbstractKursAnalyse#getResult(java.util.Date, java.util.Date)
	 */
	@Override
	public XYPlot getResult(final Date fromDate, final Date toDate) {
		if (anaG == null) {

			final DateAxis dateAxisKurs = new DateAxis(Settings.getI18n().tr(
					"RelativeStrengthIndex.axisDate")); //$NON-NLS-1$
			final NumberAxis rangeAxisKurs = new NumberAxis("RSI"); //$NON-NLS-1$
			// rangeAxisKurs.setAutoRangeIncludesZero(false);
			rangeAxisKurs.setLowerBound(0);
			rangeAxisKurs.setUpperBound(100);
			anaG = new XYPlot(null, dateAxisKurs, rangeAxisKurs, null);
			((DateAxis) anaG.getDomainAxis()).setTimeline(SegmentedTimeline
					.newMondayThroughFridayTimeline());
			anaG.setRangeAxisLocation(AxisLocation.BOTTOM_OR_LEFT);

			rendererRsi = new StandardXYItemRenderer();
			rendererRsi.setSeriesPaint(0, Color.BLACK);

			final Color lightgreen = new Color(170, 237, 169, 255);
			final Color lightred = new Color(238, 170, 169, 255);
			final Color yellow = new Color(254, 216, 61, 255);
			final Color black = new Color(0, 0, 0, 0);
			diff1 = new XYDifferenceRenderer(lightgreen, black, false);
			diff1.setSeriesStroke(0, new BasicStroke(2.0f));
			diff1.setSeriesStroke(1, new BasicStroke(1.0f));
			diff1.setSeriesPaint(0, yellow);
			diff1.setSeriesPaint(1, Color.lightGray);

			diff2 = new XYDifferenceRenderer(black, lightred, false);
			diff2.setSeriesStroke(0, new BasicStroke(2.0f));
			diff2.setSeriesStroke(1, new BasicStroke(0.0f));
			diff2.setSeriesPaint(0, yellow);
			diff2.setSeriesPaint(1, Color.lightGray);

			final LegendTitle legendTitle = new LegendTitle(anaG);
			legendTitle.setItemFont(new Font("Dialog", Font.PLAIN, 9)); //$NON-NLS-1$
			legendTitle.setBackgroundPaint(new Color(200, 200, 255, 100));
			legendTitle.setFrame(new BlockBorder(Color.white));
			legendTitle.setPosition(RectangleEdge.BOTTOM);
			final RectangleInsets padding = new RectangleInsets(4, 2, 0, 10);
			legendTitle.setItemLabelPadding(padding);
			final XYTitleAnnotation titleAnnotation = new XYTitleAnnotation(
					1.0, 0.00, legendTitle, RectangleAnchor.BOTTOM_RIGHT);

			titleAnnotation.setMaxWidth(0.48);
			anaG.addAnnotation(titleAnnotation);
		}

		if (serieRsi == null) {
			calculate();
		}

		final Day fromDay = new Day(fromDate);
		final Day toDay = new Day(toDate);

		try {

			final TimeSeriesCollection col1 = new TimeSeriesCollection();
			col1.addSeries(serieRsi.createCopy(fromDay, toDay));
			col1.addSeries(serie70.createCopy(fromDay, toDay));

			final TimeSeriesCollection col2 = new TimeSeriesCollection();
			col2.addSeries(serieRsi.createCopy(fromDay, toDay));
			col2.addSeries(serie30.createCopy(fromDay, toDay));

			anaG.setDataset(0, col1);
			anaG.setRenderer(0, diff1);
			anaG.setDataset(1, col2);
			anaG.setRenderer(1, diff2);

			diff1.setSeriesVisibleInLegend(1, false);
			diff2.setSeriesVisibleInLegend(0, false);
			diff2.setSeriesVisibleInLegend(1, false);

		} catch (CloneNotSupportedException e) {
			Logger.error(
					Settings.getI18n().tr(
							"RelativeStrengthIndex.createTsDataError"), e); //$NON-NLS-1$
		}

		return anaG;

	}

	/**
	 * Hier werden die Serien-Daten berechnet
	 */
	private void calculate() {

		final int period = parameter.getValue().intValue();
		serieRsi = new TimeSeries(getLegendTitel());
		serie30 = new TimeSeries("30"); //$NON-NLS-1$
		serie70 = new TimeSeries("70"); //$NON-NLS-1$

		for (int t = closeDaten.size() - 1; t > period; t--) {

			double sumUp1 = 0.0;
			double sumDown1 = 0.0;

			for (int i = 1; i <= period; i++) {
				sumUp1 += Math.max(
						(closeDaten.get(t - i + 1).getValue() - closeDaten.get(
								t - i).getValue()), 0);
				sumDown1 -= Math.min(
						(closeDaten.get(t - i + 1).getValue() - closeDaten.get(
								t - i).getValue()), 0);
			}

			final double avgUp = sumUp1 / period;
			final double avgDown = sumDown1 / period;
			final double rsi = avgUp / (avgUp + avgDown) * 100;

			if (rsi >= RSI_ABOVE_100) {
				Logger.info("RSI: " + rsi); //$NON-NLS-1$
			}

			final Date date = closeDaten.get(t).getDate();
			final Day day = new Day(date); // NOPMD by papa on 06.11.16 17:16
			serieRsi.add(day, rsi);
			serie30.add(day, 30);
			serie70.add(day, 70);
		}
	}

}
